@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_attr.c,v 1.63.2.1 2006/05/04 22:22:33 brad Exp $ */
+/*	$OpenBSD: rde_attr.c,v 1.66 2006/05/27 15:39:56 claudio Exp $ */
 
 /*
  * Copyright (c) 2004 Claudio Jeker <claudio@openbsd.org>
@@ -29,8 +29,6 @@
 #include "bgpd.h"
 #include "rde.h"
 
-void	attr_free(struct rde_aspath *, struct attr *);
-
 int
 attr_write(void *p, u_int16_t p_len, u_int8_t flags, u_int8_t type,
     void *data, u_int16_t data_len)
@@ -801,9 +799,11 @@ community_delete(struct rde_aspath *asp, int as, int type)
 		etype <<= 8;
 		etype |= *p++;
 
-		if (as != COMMUNITY_ANY && (u_int16_t)as != eas &&
-		    type != COMMUNITY_ANY && (u_int16_t)type != etype)
-			len += 4;
+		if ((as == COMMUNITY_ANY || (u_int16_t)as == eas) &&
+		    (type == COMMUNITY_ANY || (u_int16_t)type == etype))
+			/* match */
+			continue;
+		len += 4;
 	}
 
 	if (len == 0) {
@@ -823,13 +823,14 @@ community_delete(struct rde_aspath *asp, int as, int type)
 		etype <<= 8;
 		etype |= *p++;
 
-		if (as != COMMUNITY_ANY && (u_int16_t)as != eas &&
-		    type != COMMUNITY_ANY && (u_int16_t)type != etype) {
-			n[l++] = eas >> 8;
-			n[l++] = eas & 0xff;
-			n[l++] = etype >> 8;
-			n[l++] = etype & 0xff;
-		}
+		if ((as == COMMUNITY_ANY || (u_int16_t)as == eas) &&
+		    (type == COMMUNITY_ANY || (u_int16_t)type == etype))
+			/* match */
+			continue;
+		n[l++] = eas >> 8;
+		n[l++] = eas & 0xff;
+		n[l++] = etype >> 8;
+		n[l++] = etype & 0xff;
 	}
 
 	f = attr->flags;
